@@ -1,15 +1,18 @@
 // ***************************************************************** -*- C++ -*-
-// convert-test.cpp, $Rev: 1398 $
+// convert-test.cpp
 // Conversion test driver - make sure you have a copy of the input file around!
 
-#include <exiv2/image.hpp>
-#include <exiv2/convert.hpp>
+#include <exiv2/exiv2.hpp>
+
 #include <iostream>
 #include <iomanip>
 #include <cassert>
 
 int main(int argc, char* const argv[])
 try {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     if (argc != 2) {
         std::cout << "Usage: " << argv[0] << " file\n";
         return 1;
