@@ -1,5 +1,5 @@
 // ***************************************************************** -*- C++ -*-
-// easyaccess-test.cpp, $Rev: 3090 $
+// easyaccess-test.cpp
 // Sample program using high-level metadata access functions
 
 // included header files
@@ -43,6 +43,9 @@ static const EasyAccess easyAccess[] = {
 
 int main(int argc, char **argv)
 try {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     if (argc != 2) {
         std::cout << "Usage: " << argv[0] << " file\n";
         return 1;
