@@ -1,5 +1,5 @@
 // ***************************************************************** -*- C++ -*-
-// exifvalue.cpp, $Rev: 3090 $
+// exifvalue.cpp
 // Sample program to print value of an exif key in an image
 
 #include <exiv2/exiv2.hpp>
@@ -11,6 +11,9 @@
 
 int main(int argc, char* const argv[])
 {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     if (argc != 3) {
         std::cerr << "Usage: " << argv[0] << " file key\n";
         return 1;
