@@ -1,15 +1,18 @@
 // ***************************************************************** -*- C++ -*-
-// iptceasy.cpp, $Rev: 1703 $
+// iptceasy.cpp
 // The quickest way to access, set or modify IPTC metadata.
 
-#include <exiv2/iptc.hpp>
-#include <exiv2/image.hpp>
+#include <exiv2/exiv2.hpp>
+
 #include <iostream>
 #include <iomanip>
 #include <cassert>
 
 int main(int argc, char* const argv[])
 try {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     if (argc != 2) {
         std::cout << "Usage: " << argv[0] << " file\n";
         return 1;
