@@ -3,7 +3,6 @@
   Abstract : Key unit tests
 
   File     : key-test.cpp
-  Version  : $Rev: 2348 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History  : 24-Aug-04, ahu: created
 
@@ -20,6 +19,9 @@ using namespace Exiv2;
 
 int main()
 {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     int tc = 0;
     int rc = 0;
 
