@@ -1,5 +1,5 @@
 // ***************************************************************** -*- C++ -*-
-// mmap-test.cpp, $Rev$
+// mmap-test.cpp
 // Simple mmap tests
 
 #include <exiv2/exiv2.hpp>
@@ -11,6 +11,9 @@ using namespace Exiv2;
 
 int main(int argc, char* const argv[])
 try {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     if (argc != 2) {
         std::cout << "Usage: " << argv[0] << " file\n";
         return 1;
@@ -20,11 +23,11 @@ try {
     FileIo file(path);
     // Open the file in read mode
     if (file.open("rb") != 0) {
-        throw Error(10, path, "rb", strError());
+        throw Error(kerFileOpenFailed, path, "rb", strError());
     }
     // Map it to memory
     const Exiv2::byte* pData = file.mmap();
-    long size = file.size();
+    long size = (long)file.size();
     DataBuf buf(size);
     // Read from the memory mapped region
     memcpy(buf.pData_, pData, buf.size_);
