@@ -1,5 +1,5 @@
 // ***************************************************************** -*- C++ -*-
-// prevtest.cpp, $Rev: 3090 $
+// prevtest.cpp
 // Test access to preview images
 
 #include <exiv2/exiv2.hpp>
@@ -10,6 +10,9 @@
 
 int main(int argc, char* const argv[])
 try {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     if (argc != 2) {
         std::cout << "Usage: " << argv[0] << " file\n";
         return 1;
