@@ -1,5 +1,5 @@
 // ***************************************************************** -*- C++ -*-
-// stringto-test.cpp, $Rev: 3090 $
+// stringto-test.cpp
 // Test conversions from string to long, float and Rational types.
 
 #include <exiv2/exiv2.hpp>
@@ -40,6 +40,9 @@ const char* testcases[] = {
 
 int main()
 {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     std::cout << std::setfill(' ');
 
     std::cout << std::setw(12) << std::left << "string";
