@@ -1,5 +1,5 @@
 // ***************************************************************** -*- C++ -*-
-// werror-test.cpp, $Rev: 3513 $
+// werror-test.cpp
 // Simple tests for the wide-string error class WError
 
 #include <exiv2/exiv2.hpp>
@@ -8,8 +8,11 @@
 
 int main()
 {
+    Exiv2::XmpParser::initialize();
+    ::atexit(Exiv2::XmpParser::terminate);
+
     try {
-        throw Exiv2::Error(-1, "ARG1", "ARG2", "ARG3");
+        throw Exiv2::Error(Exiv2::kerGeneralError, "ARG1", "ARG2", "ARG3");
     }
     catch (const Exiv2::Error& e) {
         std::cout << "Caught Error '" << e.what() << "'\n";
@@ -17,7 +20,7 @@ int main()
 
 #ifdef EXV_UNICODE_PATH
     try {
-        throw Exiv2::WError(-1, L"WARG1", L"WARG2", L"WARG3");
+        throw Exiv2::WError(Exiv2::kerGeneralError, L"WARG1", L"WARG2", L"WARG3");
     }
     catch (const Exiv2::WError& e) {
         std::wstring wmsg = e.wwhat();
