@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2012 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -22,7 +21,6 @@
   @file    actions.hpp
   @brief   Implements base class Task, TaskFactory and the various supported
            actions (derived from Task).
-  @version $Rev: 2681 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Dec-03, ahu: created
@@ -32,15 +30,7 @@
 
 // *****************************************************************************
 // included header files
-
-// + standard includes
-#include <string>
-#include <map>
-
 #include "exiv2app.hpp"
-#include "image.hpp"
-#include "exif.hpp"
-#include "iptc.hpp"
 
 // *****************************************************************************
 // class declarations
@@ -134,7 +124,7 @@ namespace Action {
           prototype is replaced.
 
           @param type Task type.
-          @param task Pointer to the prototype. Ownership is transfered to the
+          @param task Pointer to the prototype. Ownership is transferred to the
                  task factory. That's what the auto pointer indicates.
         */
         void registerTask(TaskType type, Task::AutoPtr task);
@@ -172,10 +162,12 @@ namespace Action {
         int printList();
         //! Return true if key should be printed, else false
         bool grepTag(const std::string& key);
+        //! Return true if key should be printed, else false
+        bool keyTag(const std::string& key);
         //! Print all metadata in a user defined format
         int printMetadata(const Exiv2::Image* image);
-        //! Print a metadatum in a user defined format
-        void printMetadatum(const Exiv2::Metadatum& md, const Exiv2::Image* image);
+        //! Print a metadatum in a user defined format, return true if something was printed
+        bool printMetadatum(const Exiv2::Metadatum& md, const Exiv2::Image* image);
         //! Print the label for a summary line
         void printLabel(const std::string& label) const;
         /*!
@@ -270,6 +262,11 @@ namespace Action {
           @brief Erase XMP packet from the file.
          */
         int eraseXmpData(Exiv2::Image* image) const;
+        /*!
+          @brief Erase ICCProfile from the file.
+         */
+        int eraseIccProfile(Exiv2::Image* image) const;
+
 
     private:
         virtual Erase* clone_() const;
@@ -305,6 +302,10 @@ namespace Action {
                  depending on the format of the Exif thumbnail image.
          */
         void writePreviewFile(const Exiv2::PreviewImage& pvImg, int num) const;
+        /*!
+          @brief Write embedded iccProfile files.
+         */
+        int writeIccProfile(const std::string& path) const;
 
     private:
         virtual Extract* clone_() const;
@@ -328,12 +329,24 @@ namespace Action {
                  filename (\em path) minus its suffix plus "-thumb.jpg".
          */
         int insertThumbnail(const std::string& path) const;
+
         /*!
-          @brief Insert an XMP packet from a file into file \em path.
-                 The filename of the XMP packet is expected to be the image
-                 filename (\em path) minus its suffix plus ".xmp".
+          @brief Insert an XMP packet from a xmpPath into file \em path.
          */
-        int insertXmpPacket(const std::string& path) const;
+        int insertXmpPacket(const std::string& path,const std::string& xmpPath) const;
+        /*!
+          @brief Insert xmp from a DataBuf into file \em path.
+         */
+        int insertXmpPacket(const std::string& path,const Exiv2::DataBuf& xmpBlob,bool usePacket=false) const;
+
+        /*!
+          @brief Insert an ICC profile from iccPath into file \em path.
+         */
+        int insertIccProfile(const std::string& path,const std::string& iccPath) const;
+        /*!
+          @brief Insert an ICC profile from binary DataBuf into file \em path.
+         */
+        int insertIccProfile(const std::string& path,Exiv2::DataBuf& iccProfileBlob) const;
 
     private:
         virtual Insert* clone_() const;
@@ -356,7 +369,7 @@ namespace Action {
 
     private:
         virtual Modify* clone_() const;
-        //! Copy contructor needed because of AutoPtr member
+        //! Copy constructor needed because of AutoPtr member
         Modify(const Modify& /*src*/) : Task() {}
 
         //! Add a metadatum to \em pImage according to \em modifyCmd
