@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2015 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -23,7 +22,6 @@
   @brief   Canon makernote tags.<BR>References:<BR>
            [1] <a href="http://www.burren.cx/david/canon.html">EXIF MakerNote of Canon</a> by David Burren<br>
            [2] <a href="http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/Canon.html">Canon makernote tags</a> by Phil Harvey
-  @version $Rev: 3777 $
   @author  <a href="mailto:ahuggel@gmx.net">Andreas Huggel (ahu)</a><br>
            <a href="mailto:david@edeca.net">David Cannings (dc)</a>
            <a href="mailto:andi.clemens@gmx.net">Andi Clemens (ac)</a>
@@ -69,6 +67,8 @@ namespace Exiv2 {
         static const TagInfo* tagListCf();
         //! Return read-only list of built-in Canon Picture Info tags
         static const TagInfo* tagListPi();
+        //! Return read-only list of built-in Canon Time Info tags
+        static const TagInfo* tagListTi();
         //! Return read-only list of built-in Canon File Info tags
         static const TagInfo* tagListFi();
         //! Return read-only list of built-in Canon Processing Info tags
@@ -90,12 +90,18 @@ namespace Exiv2 {
         static std::ostream& printCsLensType(std::ostream& os, const Value& value, const ExifData* metadata);
         //! Camera lens information
         static std::ostream& printCsLens(std::ostream& os, const Value& value, const ExifData*);
+        //! AutoISO speed used
+        static std::ostream& printSi0x0001(std::ostream& os, const Value& value, const ExifData*);
         //! ISO speed used
         static std::ostream& printSi0x0002(std::ostream& os, const Value& value, const ExifData*);
         //! MeasuredEV
         static std::ostream& printSi0x0003(std::ostream& os, const Value& value, const ExifData*);
         //! Sequence number
         static std::ostream& printSi0x0009(std::ostream& os, const Value& value, const ExifData*);
+        //! Ambient Temperature
+        static std::ostream& printSi0x000c(std::ostream& os, const Value& value, const ExifData*);
+        //! Flash Guide Number
+        static std::ostream& printSi0x000d(std::ostream& os, const Value& value, const ExifData*);
         //! AF point used
         static std::ostream& printSi0x000e(std::ostream& os, const Value& value, const ExifData* pExifData);
         //! Subject distance
@@ -106,6 +112,8 @@ namespace Exiv2 {
         static std::ostream& printSi0x0016(std::ostream& os, const Value& value, const ExifData*);
         //! MeasuredEV2
         static std::ostream& printSi0x0017(std::ostream& os, const Value& value, const ExifData*);
+        //! Bulb Duration
+        static std::ostream& printSi0x0018(std::ostream& os, const Value& value, const ExifData*);
         //! Focus Distance
         static std::ostream& printFiFocusDistance(std::ostream& os, const Value& value, const ExifData*);
         //@}
@@ -119,6 +127,7 @@ namespace Exiv2 {
         static const TagInfo tagInfoSi_[];
         static const TagInfo tagInfoCf_[];
         static const TagInfo tagInfoPi_[];
+        static const TagInfo tagInfoTi_[];
         static const TagInfo tagInfoFi_[];
         static const TagInfo tagInfoPa_[];
         static const TagInfo tagInfoPr_[];
