@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2013 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -20,24 +19,22 @@
  */
 /*
   File:      convert.cpp
-  Version:   $Rev: 3091 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Vladimir Nadvornik (vn) <nadvornik@suse.cz>
   History:   17-Mar-08, ahu: created basic converter framework
              20-May-08, vn:  added actual conversion logic
  */
-// *****************************************************************************
-#include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: convert.cpp 3091 2013-07-24 05:15:04Z robinwmills $")
-
 // *****************************************************************************
 // included header files
+#include "config.h"
 #include "types.hpp"
+#include "error.hpp"
 #include "exif.hpp"
 #include "iptc.hpp"
-#include "xmp.hpp"
+#include "xmp_exiv2.hpp"
 #include "futils.hpp"
 #include "convert.hpp"
+#include "unused.h"
 
 // + standard includes
 #include <utility>
@@ -63,7 +60,11 @@ EXIV2_RCSID("@(#) $Id: convert.cpp 3091 2013-07-24 05:15:04Z robinwmills $")
 // Adobe XMP Toolkit
 #ifdef EXV_HAVE_XMP_TOOLKIT
 # define TXMP_STRING_TYPE std::string
-# include <XMPSDK.hpp>
+# ifdef EXV_ADOBE_XMPSDK
+#  include <XMP.hpp>
+# else
+#  include <XMPSDK.hpp>
+# endif
 # include <MD5.h>
 #endif // EXV_HAVE_XMP_TOOLKIT
 
@@ -143,6 +144,12 @@ namespace Exiv2 {
 
         //! @name Conversion functions (manipulators)
         //@{
+        /*!
+          @brief Do nothing conversion function.
+
+          Use when, for example, a one-way conversion is needed.
+         */
+        void cnvNone(const char*, const char*);
         /*!
           @brief Simple Exif to XMP conversion function.
 
@@ -322,12 +329,13 @@ namespace Exiv2 {
         { mdExif, "Exif.Image.PrimaryChromaticities",     "Xmp.tiff.PrimaryChromaticities",     &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.YCbCrCoefficients",         "Xmp.tiff.YCbCrCoefficients",         &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.ReferenceBlackWhite",       "Xmp.tiff.ReferenceBlackWhite",       &Converter::cnvExifValue, &Converter::cnvXmpValue },
-        { mdExif, "Exif.Image.DateTime",                  "Xmp.tiff.DateTime",                  &Converter::cnvExifDate , &Converter::cnvXmpDate  },
+        { mdExif, "Exif.Image.DateTime",                  "Xmp.xmp.ModifyDate",                 &Converter::cnvExifDate , &Converter::cnvXmpDate  }, // MWG Guidelines
         { mdExif, "Exif.Image.ImageDescription",          "Xmp.dc.description",                 &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.Make",                      "Xmp.tiff.Make",                      &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.Model",                     "Xmp.tiff.Model",                     &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.Software",                  "Xmp.tiff.Software",                  &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.Artist",                    "Xmp.dc.creator",                     &Converter::cnvExifValue, &Converter::cnvXmpValue },
+        { mdExif, "Exif.Image.Rating",                    "Xmp.xmp.Rating",                     &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Image.Copyright",                 "Xmp.dc.rights",                      &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Photo.ExifVersion",               "Xmp.exif.ExifVersion",               &Converter::cnvExifVersion, &Converter::cnvXmpVersion },
         { mdExif, "Exif.Photo.FlashpixVersion",           "Xmp.exif.FlashpixVersion",           &Converter::cnvExifVersion, &Converter::cnvXmpVersion },
@@ -338,8 +346,8 @@ namespace Exiv2 {
         { mdExif, "Exif.Photo.PixelYDimension",           "Xmp.exif.PixelYDimension",           &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Photo.UserComment",               "Xmp.exif.UserComment",               &Converter::cnvExifComment, &Converter::cnvXmpComment },
         { mdExif, "Exif.Photo.RelatedSoundFile",          "Xmp.exif.RelatedSoundFile",          &Converter::cnvExifValue, &Converter::cnvXmpValue },
-        { mdExif, "Exif.Photo.DateTimeOriginal",          "Xmp.exif.DateTimeOriginal",          &Converter::cnvExifDate,  &Converter::cnvXmpDate  },
-        { mdExif, "Exif.Photo.DateTimeDigitized",         "Xmp.exif.DateTimeDigitized",         &Converter::cnvExifDate,  &Converter::cnvXmpDate  },
+        { mdExif, "Exif.Photo.DateTimeOriginal",          "Xmp.photoshop.DateCreated",          &Converter::cnvExifDate,  &Converter::cnvXmpDate  }, // MWG Guidelines
+        { mdExif, "Exif.Photo.DateTimeDigitized",         "Xmp.xmp.CreateDate",                 &Converter::cnvExifDate,  &Converter::cnvXmpDate  }, // MWG Guidelines
         { mdExif, "Exif.Photo.ExposureTime",              "Xmp.exif.ExposureTime",              &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Photo.FNumber",                   "Xmp.exif.FNumber",                   &Converter::cnvExifValue, &Converter::cnvXmpValue },
         { mdExif, "Exif.Photo.ExposureProgram",           "Xmp.exif.ExposureProgram",           &Converter::cnvExifValue, &Converter::cnvXmpValue },
@@ -415,7 +423,8 @@ namespace Exiv2 {
         { mdIptc, "Iptc.Application2.Keywords",           "Xmp.dc.subject",                     &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.SubLocation",        "Xmp.iptc.Location",                  &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.SpecialInstructions","Xmp.photoshop.Instructions",         &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
-        { mdIptc, "Iptc.Application2.DateCreated",        "Xmp.photoshop.DateCreated",          &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
+        { mdIptc, "Iptc.Application2.DateCreated",        "Xmp.photoshop.DateCreated",          &Converter::cnvNone, &Converter::cnvXmpValueToIptc }, // FIXME to IPTC Date and IPTC Time
+        { mdIptc, "Iptc.Application2.DigitizationDate",   "Xmp.xmp.CreateDate",                 &Converter::cnvNone, &Converter::cnvXmpValueToIptc }, // FIXME to IPTC Date and IPTC Time
         { mdIptc, "Iptc.Application2.Byline",             "Xmp.dc.creator",                     &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.BylineTitle",        "Xmp.photoshop.AuthorsPosition",      &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
         { mdIptc, "Iptc.Application2.City",               "Xmp.photoshop.City",                 &Converter::cnvIptcValue, &Converter::cnvXmpValueToIptc },
@@ -464,6 +473,11 @@ namespace Exiv2 {
         }
     }
 
+    void Converter::cnvNone(const char*, const char*)
+    {
+        return;
+    }
+
     bool Converter::prepareExifTarget(const char* to, bool force)
     {
         Exiv2::ExifData::iterator pos = exifData_->findKey(ExifKey(to));
@@ -1049,11 +1063,10 @@ namespace Exiv2 {
         double deg = 0.0;
         double min = 0.0;
         double sec = 0.0;
-        char ref  = '\0';
+        char ref  = value[value.length() - 1];
         char sep1 = '\0';
         char sep2 = '\0';
 
-        ref = value[value.length() - 1];
         value.erase(value.length() - 1);
 
         std::istringstream in(value);
@@ -1158,9 +1171,9 @@ namespace Exiv2 {
         if (erase_) xmpData_->erase(pos);
     }
 
+#ifdef EXV_HAVE_XMP_TOOLKIT
     std::string Converter::computeExifDigest(bool tiff)
     {
-#ifdef EXV_HAVE_XMP_TOOLKIT
         std::ostringstream res;
         MD5_CTX    context;
         unsigned char digest[16];
@@ -1189,10 +1202,14 @@ namespace Exiv2 {
             res << static_cast<int>(digest[i]);
         }
         return res.str();
-#else
-        return std::string("");
-#endif
     }
+#else
+    std::string Converter::computeExifDigest(bool)
+    {
+        return std::string("");
+    }
+#endif
+
 
     void Converter::writeExifDigest()
     {
@@ -1273,7 +1290,7 @@ namespace Exiv2 {
 
     void moveExifToXmp(ExifData& exifData, XmpData& xmpData)
     {
-        Converter converter(const_cast<ExifData&>(exifData), xmpData);
+        Converter converter(exifData, xmpData);
         converter.setErase();
         converter.cnvToXmp();
     }
@@ -1286,14 +1303,14 @@ namespace Exiv2 {
 
     void moveXmpToExif(XmpData& xmpData, ExifData& exifData)
     {
-        Converter converter(exifData, const_cast<XmpData&>(xmpData));
+        Converter converter(exifData, xmpData);
         converter.setErase();
         converter.cnvFromXmp();
     }
 
     void syncExifWithXmp(ExifData& exifData, XmpData& xmpData)
     {
-        Converter converter(exifData, const_cast<XmpData&>(xmpData));
+        Converter converter(exifData, xmpData);
         converter.syncExifWithXmp();
     }
 
@@ -1310,7 +1327,7 @@ namespace Exiv2 {
     {
         if (!iptcCharset) iptcCharset = iptcData.detectCharset();
         if (!iptcCharset) iptcCharset = "ISO-8859-1";
-        Converter converter(const_cast<IptcData&>(iptcData), xmpData, iptcCharset);
+        Converter converter(iptcData, xmpData, iptcCharset);
         converter.setErase();
         converter.cnvToXmp();
     }
@@ -1323,7 +1340,7 @@ namespace Exiv2 {
 
     void moveXmpToIptc(XmpData& xmpData, IptcData& iptcData)
     {
-        Converter converter(iptcData, const_cast<XmpData&>(xmpData));
+        Converter converter(iptcData, xmpData);
         converter.setErase();
         converter.cnvFromXmp();
     }
@@ -1340,6 +1357,7 @@ namespace Exiv2 {
 # ifndef SUPPRESS_WARNINGS
         EXV_WARNING << "Charset conversion required but no character mapping functionality available.\n";
 # endif
+        UNUSED(str);
 #endif
         return ret;
     }
@@ -1356,7 +1374,7 @@ namespace {
     {
         // Naive byte-swapping, I'm sure this can be done more efficiently
         if (str.size() & 1) {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
             EXV_DEBUG << "swapBytes: Size " << str.size() << " of input string is not even.\n";
 #endif
             return false;
@@ -1374,7 +1392,7 @@ namespace {
         if (str.empty()) return true;
         int len = MultiByteToWideChar(cp, 0, str.c_str(), (int)str.size(), 0, 0);
         if (len == 0) {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
             EXV_DEBUG << "mb2wc: Failed to determine required size of output buffer.\n";
 #endif
             return false;
@@ -1383,7 +1401,7 @@ namespace {
         out.resize(len * 2);
         int ret = MultiByteToWideChar(cp, 0, str.c_str(), (int)str.size(), (LPWSTR)&out[0], len * 2);
         if (ret == 0) {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
             EXV_DEBUG << "mb2wc: Failed to convert the input string to a wide character string.\n";
 #endif
             return false;
@@ -1396,14 +1414,14 @@ namespace {
     {
         if (str.empty()) return true;
         if (str.size() & 1) {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
             EXV_DEBUG << "wc2mb: Size " << str.size() << " of input string is not even.\n";
 #endif
             return false;
         }
         int len = WideCharToMultiByte(cp, 0, (LPCWSTR)str.data(), (int)str.size() / 2, 0, 0, 0, 0);
         if (len == 0) {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
             EXV_DEBUG << "wc2mb: Failed to determine required size of output buffer.\n";
 #endif
             return false;
@@ -1412,7 +1430,7 @@ namespace {
         out.resize(len);
         int ret = WideCharToMultiByte(cp, 0, (LPCWSTR)str.data(), (int)str.size() / 2, (LPSTR)&out[0], len, 0, 0);
         if (ret == 0) {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
             EXV_DEBUG << "wc2mb: Failed to convert the input string to a multi byte string.\n";
 #endif
             return false;
@@ -1469,9 +1487,9 @@ namespace {
     }
 
     typedef bool (*ConvFct)(std::string& str);
-    
+
     struct ConvFctList {
-        bool operator==(std::pair<const char*, const char*> fromTo) const
+        bool operator==(const std::pair<const char*, const char*> &fromTo) const
             { return 0 == strcmp(from_, fromTo.first) && 0 == strcmp(to_, fromTo.second); }
         const char* from_;
         const char* to_;
@@ -1532,7 +1550,7 @@ namespace {
                               &inbytesleft,
                               &outptr,
                               &outbytesleft);
-            int outbytesProduced = sizeof(outbuf) - outbytesleft;
+            const size_t outbytesProduced = sizeof(outbuf) - outbytesleft;
             if (rc == size_t(-1) && errno != E2BIG) {
 #ifndef SUPPRESS_WARNINGS
                 EXV_WARNING << "iconv: " << strError()
@@ -1563,9 +1581,9 @@ namespace {
                 value = pos->toString();
                 if (   pos->value().ok()
                     && value.length() > 5 && value.substr(0, 5) == "lang=") {
-                    std::string::size_type pos = value.find_first_of(' ');
-                    if (pos != std::string::npos) {
-                        value = value.substr(pos + 1);
+                    const std::string::size_type first_space_pos = value.find_first_of(' ');
+                    if (first_space_pos != std::string::npos) {
+                        value = value.substr(first_space_pos + 1);
                     }
                     else {
                         value.clear();
