@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -20,25 +19,14 @@
  */
 /*
   File:      cr2image.cpp
-  Version:   $Rev: 2453 $
-  Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
-  History:   22-Apr-06, ahu: created
-
  */
-// *****************************************************************************
-#include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: cr2image.cpp 2453 2011-02-13 14:08:44Z ahuggel $")
-
 // *****************************************************************************
 // included header files
-#ifdef _MSC_VER
-# include "exv_msvc.h"
-#else
-# include "exv_conf.h"
-#endif
+#include "config.h"
 
 #include "cr2image.hpp"
-#include "cr2image_int.hpp"
+#include "tiffimage.hpp"
+#include "cr2header_int.hpp"
 #include "tiffcomposite_int.hpp"
 #include "tiffimage_int.hpp"
 #include "image.hpp"
@@ -59,7 +47,7 @@ namespace Exiv2 {
     using namespace Internal;
 
     Cr2Image::Cr2Image(BasicIo::AutoPtr io, bool /*create*/)
-        : Image(ImageType::cr2, mdExif | mdIptc, io)
+        : Image(ImageType::cr2, mdExif | mdIptc | mdXmp, io)
     {
     } // Cr2Image::Cr2Image
 
@@ -86,51 +74,67 @@ namespace Exiv2 {
         return 0;
     }
 
-    void Cr2Image::setExifData(const ExifData& /*exifData*/)
+    void Cr2Image::printStructure(std::ostream& out, Exiv2::PrintStructureOption option,int depth)
     {
-        // Todo: implement me!
-        throw(Error(32, "Exif metadata", "CR2"));
-    }
-
-    void Cr2Image::setIptcData(const IptcData& /*iptcData*/)
-    {
-        // Todo: implement me!
-        throw(Error(32, "IPTC metadata", "CR2"));
+        if (io_->open() != 0) throw Error(kerDataSourceOpenFailed, io_->path(), strError());
+        io_->seek(0,BasicIo::beg);
+        printTiffStructure(io(),out,option,depth-1);
     }
 
     void Cr2Image::setComment(const std::string& /*comment*/)
     {
         // not supported
-        throw(Error(32, "Image comment", "CR2"));
+        throw(Error(kerInvalidSettingForImage, "Image comment", "CR2"));
     }
 
     void Cr2Image::readMetadata()
     {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
         std::cerr << "Reading CR2 file " << io_->path() << "\n";
 #endif
         if (io_->open() != 0) {
-            throw Error(9, io_->path(), strError());
+            throw Error(kerDataSourceOpenFailed, io_->path(), strError());
         }
         IoCloser closer(*io_);
         // Ensure that this is the correct image type
         if (!isCr2Type(*io_, false)) {
-            if (io_->error() || io_->eof()) throw Error(14);
-            throw Error(3, "CR2");
+            if (io_->error() || io_->eof()) throw Error(kerFailedToReadImageData);
+            throw Error(kerNotAnImage, "CR2");
         }
         clearMetadata();
         ByteOrder bo = Cr2Parser::decode(exifData_,
                                          iptcData_,
                                          xmpData_,
                                          io_->mmap(),
-                                         io_->size());
+                                         (uint32_t) io_->size());
         setByteOrder(bo);
     } // Cr2Image::readMetadata
 
     void Cr2Image::writeMetadata()
     {
-        // Todo: implement me!
-        throw(Error(31, "CR2"));
+#ifdef EXIV2_DEBUG_MESSAGES
+        std::cerr << "Writing CR2 file " << io_->path() << "\n";
+#endif
+        ByteOrder bo = byteOrder();
+        byte* pData = 0;
+        long size = 0;
+        IoCloser closer(*io_);
+        if (io_->open() == 0) {
+            // Ensure that this is the correct image type
+            if (isCr2Type(*io_, false)) {
+                pData = io_->mmap(true);
+                size = (long) io_->size();
+                Cr2Header cr2Header;
+                if (0 == cr2Header.read(pData, 16)) {
+                    bo = cr2Header.byteOrder();
+                }
+            }
+        }
+        if (bo == invalidByteOrder) {
+            bo = littleEndian;
+        }
+        setByteOrder(bo);
+        Cr2Parser::encode(*io_, pData, size, bo, exifData_, iptcData_, xmpData_); // may throw
     } // Cr2Image::writeMetadata
 
     ByteOrder Cr2Parser::decode(
@@ -153,27 +157,45 @@ namespace Exiv2 {
     }
 
     WriteMethod Cr2Parser::encode(
-              Blob&     blob,
-        const byte*     /*pData*/,
-              uint32_t  /*size*/,
-        const ExifData& /*exifData*/,
-        const IptcData& /*iptcData*/,
-        const XmpData&  /*xmpData*/
+              BasicIo&  io,
+        const byte*     pData,
+              uint32_t  size,
+              ByteOrder byteOrder,
+        const ExifData& exifData,
+        const IptcData& iptcData,
+        const XmpData&  xmpData
     )
     {
-        /* Todo: Implement me!
+        // Copy to be able to modify the Exif data
+        ExifData ed = exifData;
 
-        TiffParserWorker::encode(blob,
-                                 pData,
-                                 size,
-                                 exifData,
-                                 iptcData,
-                                 xmpData,
-                                 TiffCreator::create,
-                                 TiffMapping::findEncoder);
-        */
-        blob.clear();
-        return wmIntrusive;
+        // Delete IFDs which do not occur in TIFF images
+        static const IfdId filteredIfds[] = {
+            panaRawId
+        };
+        for (unsigned int i = 0; i < EXV_COUNTOF(filteredIfds); ++i) {
+#ifdef EXIV2_DEBUG_MESSAGES
+            std::cerr << "Warning: Exif IFD " << filteredIfds[i] << " not encoded\n";
+#endif
+            ed.erase(std::remove_if(ed.begin(),
+                                    ed.end(),
+                                    FindExifdatum(filteredIfds[i])),
+                     ed.end());
+        }
+
+        std::auto_ptr<TiffHeaderBase> header(new Cr2Header(byteOrder));
+        OffsetWriter offsetWriter;
+        offsetWriter.setOrigin(OffsetWriter::cr2RawIfdOffset, Cr2Header::offset2addr(), byteOrder);
+        return TiffParserWorker::encode(io,
+                                        pData,
+                                        size,
+                                        ed,
+                                        iptcData,
+                                        xmpData,
+                                        Tag::root,
+                                        TiffMapping::findEncoder,
+                                        header.get(),
+                                        &offsetWriter);
     }
 
     // *************************************************************************
@@ -204,47 +226,3 @@ namespace Exiv2 {
     }
 
 }                                       // namespace Exiv2
-
-namespace Exiv2 {
-    namespace Internal {
-
-    const char* Cr2Header::cr2sig_ = "CR\2\0";
-
-    Cr2Header::Cr2Header()
-        : TiffHeaderBase(42, 16, littleEndian, 0x00000010),
-          offset2_(0x00000000)
-    {
-    }
-
-    Cr2Header::~Cr2Header()
-    {
-    }
-
-    bool Cr2Header::read(const byte* pData, uint32_t size)
-    {
-        if (size < 16) return false;
-
-        if (pData[0] == 0x49 && pData[1] == 0x49) {
-            setByteOrder(littleEndian);
-        }
-        else if (pData[0] == 0x4d && pData[1] == 0x4d) {
-            setByteOrder(bigEndian);
-        }
-        else {
-            return false;
-        }
-        if (tag() != getUShort(pData + 2, byteOrder())) return false;
-        setOffset(getULong(pData + 4, byteOrder()));
-        if (0 != memcmp(pData + 8, cr2sig_, 4)) return false;
-        offset2_ = getULong(pData + 12, byteOrder());
-
-        return true;
-    } // Cr2Header::read
-
-    DataBuf Cr2Header::write() const
-    {
-        // Todo: Implement me!
-        return DataBuf();
-    }
-
-}}                                      // namespace Internal, Exiv2
