@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2013 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -20,22 +19,13 @@
  */
 /*
   File:      exif.cpp
-  Version:   $Rev: 3091 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
-// *****************************************************************************
-#include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: exif.cpp 3091 2013-07-24 05:15:04Z robinwmills $")
-
 // *****************************************************************************
 // included header files
-#ifdef _MSC_VER
-# include "exv_msvc.h"
-#else
-# include "exv_conf.h"
-#endif
+#include "config.h"
 
 #include "exif.hpp"
 #include "metadatum.hpp"
@@ -229,15 +219,24 @@ namespace Exiv2 {
     std::ostream& Exifdatum::write(std::ostream& os, const ExifData* pMetadata) const
     {
         if (value().count() == 0) return os;
-        PrintFct fct = printValue;
-        const TagInfo* ti = Internal::tagInfo(tag(), static_cast<IfdId>(ifdId()));
-        if (ti != 0) fct = ti->printFct_;
-        return fct(os, value(), pMetadata);
+
+        PrintFct       fct = printValue;
+        const TagInfo* ti  = Internal::tagInfo(tag(), static_cast<IfdId>(ifdId()));
+        // be careful with comments (User.Photo.UserComment, GPSAreaInfo etc).
+        if ( ti ) {
+            fct = ti->printFct_;
+            if ( ti->typeId_ == comment ) {
+              os << value().toString();
+              fct=NULL;
+            }
+        }
+        if ( fct ) fct(os, value(), pMetadata);
+        return os;
     }
 
     const Value& Exifdatum::value() const
     {
-        if (value_.get() == 0) throw Error(8);
+        if (value_.get() == 0) throw Error(kerValueNotSet);
         return *value_;
     }
 
@@ -670,12 +669,28 @@ namespace Exiv2 {
             "Exif.Image.StripByteCounts",
             "Exif.Image.JPEGInterchangeFormat",
             "Exif.Image.JPEGInterchangeFormatLength",
-            "Exif.Image.SubIFDs"
+            "Exif.Image.SubIFDs",
+            // Issue 981.  Never allow manufactured data to be written
+            "Exif.Canon.AFInfoSize",
+            "Exif.Canon.AFAreaMode",
+            "Exif.Canon.AFNumPoints",
+            "Exif.Canon.AFValidPoints",
+            "Exif.Canon.AFCanonImageWidth",
+            "Exif.Canon.AFCanonImageHeight",
+            "Exif.Canon.AFImageWidth",
+            "Exif.Canon.AFImageHeight",
+            "Exif.Canon.AFAreaWidths",
+            "Exif.Canon.AFAreaHeights",
+            "Exif.Canon.AFXPositions",
+            "Exif.Canon.AFYPositions",
+            "Exif.Canon.AFPointsInFocus",
+            "Exif.Canon.AFPointsSelected",
+            "Exif.Canon.AFPointsUnusable",
         };
         for (unsigned int i = 0; i < EXV_COUNTOF(filteredIfd0Tags); ++i) {
             ExifData::iterator pos = ed.findKey(ExifKey(filteredIfd0Tags[i]));
             if (pos != ed.end()) {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
                 std::cerr << "Warning: Exif tag " << pos->key() << " not encoded\n";
 #endif
                 ed.erase(pos);
@@ -699,7 +714,7 @@ namespace Exiv2 {
             ifd3Id
         };
         for (unsigned int i = 0; i < EXV_COUNTOF(filteredIfds); ++i) {
-#ifdef DEBUG
+#ifdef EXIV2_DEBUG_MESSAGES
             std::cerr << "Warning: Exif IFD " << filteredIfds[i] << " not encoded\n";
 #endif
             eraseIfd(ed, filteredIfds[i]);
@@ -723,7 +738,7 @@ namespace Exiv2 {
                                                   header.get(),
                                                   0);
         if (mio1.size() <= 65527) {
-            append(blob, mio1.mmap(), mio1.size());
+            append(blob, mio1.mmap(), (uint32_t) mio1.size());
             return wm;
         }
 
@@ -799,17 +814,17 @@ namespace Exiv2 {
             }
         }
 
-        // Delete unknown tags larger than 4kB and known tags larger than 40kB.
-        for (ExifData::iterator pos = ed.begin(); pos != ed.end(); ) {
-            if (   (pos->size() > 4096 && pos->tagName().substr(0, 2) == "0x")
-                || pos->size() > 40960) {
+        // Delete unknown tags larger than 4kB and known tags larger than 20kB.
+        for (ExifData::iterator tag_iter = ed.begin(); tag_iter != ed.end(); ) {
+            if ( (tag_iter->size() > 4096 && tag_iter->tagName().substr(0, 2) == "0x") ||
+                  tag_iter->size() > 20480) {
 #ifndef SUPPRESS_WARNINGS
-                EXV_WARNING << "Exif tag " << pos->key() << " not encoded\n";
+                EXV_WARNING << "Exif tag " << tag_iter->key() << " not encoded\n";
 #endif
-                pos = ed.erase(pos);
+                tag_iter = ed.erase(tag_iter);
             }
             else {
-                ++pos;
+                ++tag_iter;
             }
         }
 
@@ -825,8 +840,8 @@ namespace Exiv2 {
                                       TiffMapping::findEncoder,
                                       header.get(),
                                       0);
-        append(blob, mio2.mmap(), mio2.size());
-#ifdef DEBUG
+        append(blob, mio2.mmap(), (uint32_t) mio2.size());
+#ifdef EXIV2_DEBUG_MESSAGES
         if (wm == wmIntrusive) {
             std::cerr << "SIZE OF EXIF DATA IS " << std::dec << mio2.size() << " BYTES\n";
         }
@@ -902,7 +917,7 @@ namespace {
         Exiv2::IptcData emptyIptc;
         Exiv2::XmpData  emptyXmp;
         Exiv2::TiffParser::encode(io, 0, 0, Exiv2::littleEndian, thumb, emptyIptc, emptyXmp);
-        return io.read(io.size());
+        return io.read((long) io.size());
     }
 
     const char* JpegThumbnail::mimeType() const
