@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2015 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -18,66 +17,85 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
-/*
-  File:      futils.cpp
-  Version:   $Rev: 3777 $
-  Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
-  History:   08-Dec-03, ahu: created
-             02-Apr-05, ahu: moved to Exiv2 namespace
- */
 // *****************************************************************************
-#include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: futils.cpp 3777 2015-05-02 11:55:40Z ahuggel $")
-
 // included header files
 #include "config.h"
 
 #include "futils.hpp"
+#include "datasets.hpp"
+#include "enforce.hpp"
+#include "image_int.hpp"
 
 // + standard includes
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef _MSC_VER
-# define S_ISREG(m)      (((m) & S_IFMT) == S_IFREG)
-#endif
-#ifdef EXV_HAVE_UNISTD_H
-# include <unistd.h>                     // for stat()
-#endif
-
+#include <cstdio>
 #include <cerrno>
 #include <sstream>
 #include <cstring>
 #include <algorithm>
+#include <stdexcept>
+#ifdef   EXV_HAVE_UNISTD_H
+#include <unistd.h>                     // for stat()
+#endif
 
-#if defined EXV_HAVE_STRERROR_R && !defined EXV_HAVE_DECL_STRERROR_R
-# ifdef EXV_STRERROR_R_CHAR_P
-extern char *strerror_r(int errnum, char *buf, size_t n);
-# else
-extern int strerror_r(int errnum, char *buf, size_t n);
-# endif
+#if defined(WIN32)
+#include <windows.h>
+#include <psapi.h>  // For access to GetModuleFileNameEx
+#endif
+
+#if defined(_MSC_VER)
+#define S_ISREG(m)      (((m) & S_IFMT) == S_IFREG)
+#elif defined(__APPLE__)
+#include <libproc.h>
+#endif
+
+#if defined(__FreeBSD__)
+#include <sys/mount.h>
+#include <sys/param.h>
+#include <sys/queue.h>
+#include <sys/socket.h>
+#include <sys/sysctl.h>
+#include <sys/types.h>
+#include <sys/un.h>
+#include <unistd.h>
+#include <libprocstat.h>
+#endif
+
+#ifndef _MAX_PATH
+#define _MAX_PATH 1024
 #endif
 
 namespace Exiv2 {
-    const char* ENVARDEF[] = {"/exiv2.php", "40"};
-    const char* ENVARKEY[] = {"EXIV2_HTTP_POST", "EXIV2_TIMEOUT"};
+    const char* ENVARDEF[] = {"/exiv2.php", "40"}; //!< @brief default URL for http exiv2 handler and time-out
+    const char* ENVARKEY[] = {"EXIV2_HTTP_POST", "EXIV2_TIMEOUT"}; //!< @brief request keys for http exiv2 handler and time-out
 // *****************************************************************************
 // free functions
-    std::string getEnv(EnVar var) {
-        return getenv(ENVARKEY[var]) ? getenv(ENVARKEY[var]) : ENVARDEF[var];
-    } // getEnv
+    std::string getEnv(int env_var)
+    {
+        // this check is relying on undefined behavior and might not be effective
+        if (env_var < envHTTPPOST || env_var > envTIMEOUT) {
+            throw std::out_of_range("Unexpected env variable");
+        }
+        return getenv(ENVARKEY[env_var]) ? getenv(ENVARKEY[env_var]) : ENVARDEF[env_var];
+    }
 
+    /// @brief Convert an integer value to its hex character.
     char to_hex(char code) {
         static char hex[] = "0123456789abcdef";
         return hex[code & 15];
-    } // to_hex
+    }
 
+    /// @brief Convert a hex character to its integer value.
     char from_hex(char ch) {
         return isdigit(ch) ? ch - '0' : tolower(ch) - 'a' + 10;
-    } // from_hex
+    }
 
-    char* urlencode(char* str) {
-        char* pstr = str;
-        char* buf  = (char*)malloc(strlen(str) * 3 + 1);
+    std::string urlencode(const char* str) {
+        const char* pstr = str;
+        // \todo try to use std::string for buf and avoid the creation of another string for just
+        // returning the final value
+        char* buf  = new char[strlen(str) * 3 + 1];
         char* pbuf = buf;
         while (*pstr) {
             if (isalnum(*pstr) || *pstr == '-' || *pstr == '_' || *pstr == '.' || *pstr == '~')
@@ -89,12 +107,14 @@ namespace Exiv2 {
             pstr++;
         }
         *pbuf = '\0';
-        return buf;
-    } // urlencode
+        std::string ret(buf);
+        delete [] buf;
+        return ret;
+    }
 
     char* urldecode(const char* str) {
         const char* pstr = str;
-        char* buf  = (char*)malloc(strlen(str) + 1);
+        char* buf  = new char [(strlen(str) + 1)];
         char* pbuf = buf;
         while (*pstr) {
             if (*pstr == '%') {
@@ -111,13 +131,13 @@ namespace Exiv2 {
         }
         *pbuf = '\0';
         return buf;
-    } // urldecode
+    }
 
     void urldecode(std::string& str) {
         char* decodeStr = Exiv2::urldecode(str.c_str());
         str = std::string(decodeStr);
-        free(decodeStr);
-    } // urldecode(const std::string& str)
+        delete [] decodeStr;
+    }
 
     int base64encode(const void* data_buf, size_t dataLength, char* result, size_t resultSize) {
         const char base64chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
@@ -241,56 +261,62 @@ namespace Exiv2 {
     Protocol fileProtocol(const std::string& path) {
         Protocol result = pFile ;
         struct {
-        	std::string name ;
-        	Protocol    prot ;
+            std::string name ;
+            Protocol    prot ;
+            bool        isUrl; // path.size() > name.size()
         } prots[] =
-        { { "http://"   ,pHttp     }
-        , { "https://"  ,pHttps    }
-        , { "ftp://"    ,pFtp      }
-        , { "sftp://"   ,pSftp     }
-        , { "ssh://"    ,pSsh      }
-        , { "file://"   ,pFileUri  }
-        , { "data://"   ,pDataUri  }
-        , { "-"         ,pStdin    }
+        { { "http://"   ,pHttp     , true  }
+        , { "https://"  ,pHttps    , true  }
+        , { "ftp://"    ,pFtp      , true  }
+        , { "sftp://"   ,pSftp     , true  }
+        , { "ssh://"    ,pSsh      , true  }
+        , { "file://"   ,pFileUri  , true  }
+        , { "data://"   ,pDataUri  , true  }
+        , { "-"         ,pStdin    , false }
         };
         for ( size_t i = 0 ; result == pFile && i < sizeof(prots)/sizeof(prots[0]) ; i ++ )
-        	if ( path.find(prots[i].name) == 0 )
-        		result = prots[i].prot;
+            if ( path.find(prots[i].name) == 0 )
+                // URL's require data.  Stdin == "-" and no further data
+                if ( prots[i].isUrl ? path.size() > prots[i].name.size() : path.size() == prots[i].name.size() )
+                    result = prots[i].prot;
 
         return result;
     } // fileProtocol
 #ifdef EXV_UNICODE_PATH
-    Protocol fileProtocol(const std::wstring& wpath) {
+    Protocol fileProtocol(const std::wstring& path) {
         Protocol result = pFile ;
         struct {
-        	std::wstring wname ;
-        	Protocol      prot ;
+            std::wstring  name ;
+            Protocol      prot ;
+            bool          isUrl; // path.size() > name.size()
         } prots[] =
-        { { L"http://"   ,pHttp     }
-        , { L"https://"  ,pHttps    }
-        , { L"ftp://"    ,pFtp      }
-        , { L"sftp://"   ,pSftp     }
-        , { L"ssh://"    ,pSsh      }
-        , { L"file://"   ,pFileUri  }
-        , { L"data://"   ,pDataUri  }
-        , { L"-"         ,pStdin    }
+        { { L"http://"   ,pHttp     , true  }
+        , { L"https://"  ,pHttps    , true  }
+        , { L"ftp://"    ,pFtp      , true  }
+        , { L"sftp://"   ,pSftp     , true  }
+        , { L"ssh://"    ,pSsh      , true  }
+        , { L"file://"   ,pFileUri  , true  }
+        , { L"data://"   ,pDataUri  , true  }
+        , { L"-"         ,pStdin    , false }
         };
         for ( size_t i = 0 ; result == pFile && i < sizeof(prots)/sizeof(prots[0]) ; i ++ )
-        	if ( wpath.find(prots[i].wname) == 0 )
-        		result = prots[i].prot;
+            if ( path.find(prots[i].name) == 0 )
+                // URL's require data.  Stdin == "-" and no further data
+                if ( prots[i].isUrl ? path.size() > prots[i].name.size() : path.size() == prots[i].name.size() )
+                    result = prots[i].prot;
 
         return result;
     } // fileProtocol
 #endif
     bool fileExists(const std::string& path, bool ct)
     {
-		// special case: accept "-" (means stdin)
-        if (path.compare("-") == 0 || fileProtocol(path)) {
-			return true;
+        // special case: accept "-" (means stdin)
+        if (path.compare("-") == 0 || fileProtocol(path) != pFile) {
+            return true;
         }
 
         struct stat buf;
-		int ret = ::stat(path.c_str(), &buf);
+        int ret = ::stat(path.c_str(), &buf);
         if (0 != ret)                    return false;
         if (ct && !S_ISREG(buf.st_mode)) return false;
         return true;
@@ -299,9 +325,9 @@ namespace Exiv2 {
 #ifdef EXV_UNICODE_PATH
     bool fileExists(const std::wstring& wpath, bool ct)
     {
-		// special case: accept "-" (means stdin)
-        if (wpath.compare(L"-") == 0 || fileProtocol(wpath)) {
-			return true;
+        // special case: accept "-" (means stdin)
+        if (wpath.compare(L"-") == 0 || fileProtocol(wpath) != pFile) {
+            return true;
         }
 
         struct _stat buf;
@@ -326,27 +352,30 @@ namespace Exiv2 {
         else return path.substr(found);
     }
 #endif
+
     std::string strError()
     {
         int error = errno;
         std::ostringstream os;
 #ifdef EXV_HAVE_STRERROR_R
         const size_t n = 1024;
-// _GNU_SOURCE: See Debian bug #485135
-# if defined EXV_STRERROR_R_CHAR_P && defined _GNU_SOURCE
+#ifdef EXV_STRERROR_R_CHAR_P
         char *buf = 0;
         char buf2[n];
         std::memset(buf2, 0x0, n);
         buf = strerror_r(error, buf2, n);
-# else
+#else
         char buf[n];
         std::memset(buf, 0x0, n);
-        strerror_r(error, buf, n);
-# endif
+        const int ret = strerror_r(error, buf, n);
+        enforce(ret != ERANGE, Exiv2::kerCallFailed);
+#endif
         os << buf;
         // Issue# 908.
         // report strerror() if strerror_r() returns empty
-        if ( !buf[0] ) os << strerror(error);
+        if ( !buf[0] ) {
+            os << strerror(error);
+        }
 #else
         os << std::strerror(error);
 #endif
@@ -402,12 +431,12 @@ namespace Exiv2 {
             iterator_t userEnd   = std::find(authStart, authEnd, ':');
             if (userEnd != authEnd) {
                 result.Username = std::string(userStart, userEnd);
-                userEnd++;
+                ++userEnd;
                 result.Password = std::string(userEnd, authEnd);
             } else {
                 result.Username = std::string(authStart, authEnd);
             }
-            authEnd++;
+            ++authEnd;
         } else {
           authEnd = protocolEnd;
         }
@@ -425,7 +454,7 @@ namespace Exiv2 {
         // port
         if ((hostEnd != uriEnd) && ((&*(hostEnd))[0] == ':'))  // we have a port
         {
-            hostEnd++;
+            ++hostEnd;
             iterator_t portEnd = (pathStart != uriEnd) ? pathStart : queryStart;
             result.Port = std::string(hostEnd, portEnd);
         }
@@ -440,5 +469,59 @@ namespace Exiv2 {
             result.QueryString = std::string(queryStart, uri.end());
 
         return result;
-    }   // Uri::Parse
+    }
+
+    std::string getProcessPath()
+    {
+        std::string ret("unknown");
+    #if defined(WIN32)
+        HANDLE processHandle = NULL;
+        processHandle = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, GetCurrentProcessId());
+        if (processHandle != NULL) {
+            TCHAR filename[MAX_PATH];
+            if (GetModuleFileNameEx(processHandle, NULL, filename, MAX_PATH) != 0) {
+                ret = filename;
+            }
+            CloseHandle(processHandle);
+        }
+    #elif defined(__APPLE__)
+        const int pid = getpid();
+        char pathbuf[PROC_PIDPATHINFO_MAXSIZE];
+        if (proc_pidpath (pid, pathbuf, sizeof(pathbuf)) > 0) {
+            ret = pathbuf;
+        }
+    #elif defined(__FreeBSD__)
+        unsigned int       n;
+        char               buffer[PATH_MAX] = {};
+        struct procstat*   procstat = procstat_open_sysctl();
+        struct kinfo_proc* procs    = procstat ? procstat_getprocs(procstat, KERN_PROC_PID, getpid(), &n) : NULL;
+        if ( procs ) {
+            procstat_getpathname(procstat, procs, buffer, PATH_MAX);
+            ret = std::string(buffer);
+        }
+        // release resources
+        if ( procs    ) procstat_freeprocs(procstat, procs);
+        if ( procstat ) procstat_close(procstat);
+    #elif defined(__sun__)
+        // https://stackoverflow.com/questions/47472762/on-solaris-how-to-get-the-full-path-of-executable-of-running-process-programatic        
+        const char* proc = Internal::stringFormat("/proc/%d/path/a.out",getpid()).c_str();
+        char        path[500];
+        ssize_t     l = readlink (proc,path,sizeof(path)-1);
+        if (l>0) {
+            path[l]=0;
+            ret = path;
+        }
+    #elif defined(__unix__)
+        // http://stackoverflow.com/questions/606041/how-do-i-get-the-path-of-a-process-in-unix-linux
+        char path[500];
+        ssize_t l = readlink ("/proc/self/exe", path,sizeof(path)-1);
+        if (l>0) {
+            path[l]=0;
+            ret = path;
+        }
+    #endif
+
+        const size_t idxLastSeparator = ret.find_last_of(EXV_SEPARATOR_STR);
+        return ret.substr(0, idxLastSeparator);
+    }
 }                                       // namespace Exiv2
