@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2015 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +23,6 @@
            vendor specific makernote implementations.<BR>References:<BR>
   [1] <a href="http://www.sno.phy.queensu.ca/~phil/exiftool/">ExifTool</a> by Phil Harvey<BR>
   [2] <a href="http://www.cybercom.net/~dcoffin/dcraw/">Decoding raw digital photos in Linux</a> by Dave Coffin
-  @version $Rev: 3777 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Apr-06, ahu: created
@@ -36,6 +34,7 @@
 // included header files
 #include "tifffwd_int.hpp"
 #include "tags_int.hpp"
+#include "ini.hpp"
 #include "types.hpp"
 
 // + standard includes
@@ -45,6 +44,18 @@
 // namespace extensions
 namespace Exiv2 {
     namespace Internal {
+// *****************************************************************************
+// function prototypes
+        /*!
+          @brief Determine the path to the Exiv2 configuration file
+         */
+        std::string getExiv2ConfigPath();
+
+        /*!
+          @brief Read value from Exiv2 configuration file
+         */
+        std::string readExiv2Config(const std::string& section,const std::string& value,const std::string& def);
+
 
 // *****************************************************************************
 // class definitions
@@ -719,6 +730,17 @@ namespace Exiv2 {
      */
     int sonyCsSelector(uint16_t tag, const byte* pData, uint32_t size, TiffComponent* const pRoot);
 
+    /*!
+        @brief Function to select cfg + def of the Sony 2010 Miscellaneous Information complex binary array.
+
+        @param tag Tag number of the binary array
+        @param pData Pointer to the raw array data.
+        @param size Size of the array data.
+        @param pRoot Pointer to the root component of the TIFF tree.
+        @return An index into the array set, -1 if no match was found.
+    */
+    int sony2010eSelector(uint16_t tag, const byte* pData, uint32_t size, TiffComponent* const pRoot);
+       
     /*!
       @brief Function to select cfg + def of a Nikon complex binary array.
 
@@ -730,6 +752,17 @@ namespace Exiv2 {
      */
     int nikonSelector(uint16_t tag, const byte* pData, uint32_t size, TiffComponent* const pRoot);
 
+    /*!
+      @brief Function to select cfg + def of a Nikon complex binary array.
+
+      @param tag Tag number of the binary array
+      @param pData Pointer to the raw array data.
+      @param size Size of the array data.
+      @param pRoot Pointer to the root component of the TIFF tree.
+      @return An index into the array set, -1 if no match was found.
+     */
+     int nikonAf2Selector(uint16_t tag, const byte* pData, uint32_t size, TiffComponent* const pRoot);
+
     /*!
       @brief Encrypt and decrypt Nikon data.
 
