@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2015 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +23,6 @@
            <a href="http://www.sno.phy.queensu.ca/~phil/exiftool/TagNames/Sony.html">Sony Makernote list</a> by Phil Harvey<br>
 Email communication with <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a><br>
 
-  @version $Rev: 3777 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
              @author  Gilles Caulier (cgilles)
@@ -38,6 +36,7 @@ Email communication with <a href="mailto:caulier dot gilles at gmail dot com">ca
 // included header files
 #include "tags.hpp"
 #include "types.hpp"
+#include "tiffcomposite_int.hpp"
 
 // + standard includes
 #include <string>
@@ -60,6 +59,10 @@ namespace Exiv2 {
         static const TagInfo* tagListCs();
         //! Return read-only list of built-in Sony Standard Camera Settings version 2 tags
         static const TagInfo* tagListCs2();
+        //! Return read-only list of built-in Sony FocusPosition tags
+        static const TagInfo* tagListFp();
+
+        static const TagInfo* tagList2010e();
 
         //! @name Print functions for Sony %MakerNote tags
         //@{
@@ -73,9 +76,13 @@ namespace Exiv2 {
         static const TagInfo tagInfo_[];
         static const TagInfo tagInfoCs_[];
         static const TagInfo tagInfoCs2_[];
-
+        static const TagInfo tagInfoFp_[];
+        static const TagInfo tagInfo2010e_[];
     }; // class SonyMakerNote
 
+    DataBuf sonyTagDecipher(uint16_t, const byte*, uint32_t, TiffComponent* const);
+    DataBuf sonyTagEncipher(uint16_t, const byte*, uint32_t, TiffComponent* const);
+
 }}                                      // namespace Internal, Exiv2
 
 #endif                                  // #ifndef SONYMN_INT_HPP_
