@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2013 Andreas Huggel <ahuggel@gmx.net>
- *
+ * Copyright (C) 2004-2018 Exiv2 authors
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
@@ -20,35 +19,26 @@
  */
 /*
   File:      utils.cpp
-  Version:   $Rev: 3201 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
  */
-// *****************************************************************************
-#include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: utils.cpp 3201 2013-12-01 12:13:42Z ahuggel $")
-
 // *****************************************************************************
 // included header files
-#ifdef _MSC_VER
-# include "exv_msvc.h"
-#else
-# include "exv_conf.h"
-#endif
+#include "config.h"
 
 #include "utils.hpp"
 
-// + standard includes
-#include <sys/types.h>
-#include <sys/stat.h>
-#ifdef _MSC_VER
-# include "getopt_win32.h"
+#if defined(_MSC_VER)
 # define S_ISREG(m)      (((m) & S_IFMT) == S_IFREG)
 #endif
+
+// + standard includes
 #ifdef EXV_HAVE_UNISTD_H
-# include <unistd.h>                     // for getopt(), stat()
+# include <unistd.h>                     // for stat()
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <climits>
 #include <cerrno>
 #include <cstdlib>
@@ -59,37 +49,6 @@ EXIV2_RCSID("@(#) $Id: utils.cpp 3201 2013-12-01 12:13:42Z ahuggel $")
 
 namespace Util {
 
-// *****************************************************************************
-// class Getopt
-    Getopt::Getopt()
-        : errcnt_(0)
-    {
-    }
-
-    Getopt::~Getopt()
-    {
-    }
-
-    int Getopt::getopt(int argc, char* const argv[], const std::string& optstring)
-    {
-        progname_ = Util::basename(argv[0]);
-
-        for (;;) {
-            int c = ::getopt(argc, argv, optstring.c_str());
-            if (c == -1) break;
-            errcnt_ += option(c, ::optarg == 0 ? "" : ::optarg, ::optopt);
-        }
-        for (int i = ::optind; i < argc; i++) {
-            errcnt_ += nonoption(argv[i]);
-        }
-        return errcnt_;
-    }
-
-    int Getopt::nonoption(const std::string& /*argv*/)
-    {
-        return 0;
-    }
-
 // *****************************************************************************
 // free functions
 
